@@ -4,7 +4,7 @@
 modname = 'trackervcs'
 distname = 'cubicweb-trackervcs'
 
-numversion = (0, 5, 1)
+numversion = (0, 6, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -14,9 +14,11 @@ description = 'vcsfile / tracker integration'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ = {'cubicweb': '>= 3.10.0',
-               'cubicweb-vcreview': '>= 0.4.1',
+               'cubicweb-vcsfile': '>= 0.4.1',
                'cubicweb-tracker': None,
                }
+__recommends__ = {'cubicweb-vcreview': '>= 0.4.1',
+                  }
 
 
 from os import listdir as _listdir
