@@ -47,7 +47,7 @@ class SearchRevisionTicketOp(hook.DataOperationMixIn,
                     continue
                 for eid in TICKET_EID_RGX.findall(' '.join(words[idx+1:])):
                     rset = self.session.execute(
-                        'Any X WHERE X eid %(x)s, T concerns P, '
+                        'Any X WHERE X eid %(x)s, X concerns P, '
                         'P source_repository R, R eid %(r)s',
                         {'x': typed_eid(eid), 'r': rev.repository.eid})
                     if rset:
@@ -104,17 +104,58 @@ class PatchLinkedToTicket(hook.Hook):
         ticket = self._cw.entity_from_eid(self.eidto)
         ticket.cw_adapt_to('IWorkflowable').fire_transition_if_possible('start')
 
+# attach patch to ticket on addition ###########################################
+
+class PatchRevisionAdded(hook.Hook):
+    __regid__ = 'trackervcs.patch-revision-added'
+    __select__ = hook.Hook.__select__ & hook.match_rtype('patch_revision', toetypes=('VersionContent',))
+    events = ('after_add_relation',)
+    category = 'autoset'
+
+    def __call__(self):
+        patch = self._cw.entity_from_eid(self.eidfrom)
+        versioncontent = self._cw.entity_from_eid(self.eidto)
+        patch_repo = versioncontent.repository
+        source_repo = patch_repo.patchrepo_of
+        for line in versioncontent.data.readlines():
+            # skip mercurial metadata
+            if line[0] == '#':
+                continue
+            # and stop at the end of the header
+            if line[:4] == 'diff':
+                break
+            words = [w.lower() for w in re.findall('#?\w+', line.strip())]
+            try:
+                idx = words.index('closes')
+            except ValueError:
+                continue
+            for eid in TICKET_EID_RGX.findall(' '.join(words[idx+1:])):
+                rset = self._cw.execute(
+                        'Any X WHERE X eid %(x)s, X concerns P, P source_repository R, R eid %(r)s',
+                        {'x': typed_eid(eid), 'r': source_repo.eid})
+                if rset:
+                    ticket = rset.get_entity(0,0)
+                    patch.set_relations(patch_ticket=ticket)
 
 # security propagation #########################################################
 
-tracker.S_RELS |= set(('source_repository', 'has_patch_repository',
-                       'patch_revision',
-                       'has_activity', ))
-tracker.O_RELS |= set(('patch_repository', 'point_of',))
+tracker.S_RELS.add('source_repository')
 
 
 def registration_callback(vreg):
-    vreg.register_all(globals().values(), __name__, (RevisionAdded,))
+    vreg.register_all(globals().values(), __name__, (RevisionAdded,
+                                                     PatchLinkedToTicket,
+                                                     PatchRevisionAdded,
+                                                     ApycotEnvironmentAdded))
     if vreg.config['trusted-vcs-repositories']:
         vreg.register(RevisionAdded)
 
+    if 'vcreview' in vreg.config.cubes():
+        vreg.register(PatchLinkedToTicket)
+        vreg.register(PatchRevisionAdded)
+        # security propagation #################################################
+        tracker.S_RELS |= set(('has_patch_repository', 'patch_revision',
+                               'has_activity', ))
+        tracker.O_RELS |= set(('patch_repository', 'point_of',))
+    if 'apycot' in vreg.config.cubes():
+        vreg.register(ApycotEnvironmentAdded)
